package com.mh.user.controller;

import com.mh.common.http.HttpResult;
import com.mh.user.annotation.BusinessType;
import com.mh.user.annotation.SysLogger;
import com.mh.user.entity.AlarmInfoEntity;
import com.mh.user.entity.AlarmValueSetEntity;
import com.mh.user.entity.EnergyEntity;
import com.mh.user.model.BuildingModel;
import com.mh.user.service.AlarmInfoService;
import com.mh.user.service.BuildingService;
import com.mh.user.service.DealDataService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

@RestController
@RequestMapping("alarm")
public class AlarmInfoController {

    @Autowired
    AlarmInfoService alarmInfoService;

    @Autowired
    BuildingService buildingService;

    @Autowired
    DealDataService dealDataService;

    @SysLogger(title="报警信息",optDesc ="保存报修信息")
    @PostMapping("/save")
    public HttpResult saveAlarmInfo(@RequestBody AlarmInfoEntity alarmInfoEntity){
        try{
            alarmInfoService.saveAlarmInfo(alarmInfoEntity);
            return  HttpResult.ok();
        }catch (Exception e){
            return  HttpResult.error(500,"保存出错!");
        }

    }

    @SysLogger(title="报警信息",optDesc = "修改报警信息")
    @PostMapping("/update")
    public HttpResult updateAlarmInfo(@RequestBody AlarmInfoEntity alarmInfoEntity){
        try{
            alarmInfoService.updateAlarmInfo(alarmInfoEntity);
            return  HttpResult.ok();
        }catch (Exception e){
            return  HttpResult.error(500,"修改出错!");
        }

    }

    @SysLogger(title="报警信息",optDesc = "删除报警信息")
    @PostMapping("/delete")
    public HttpResult deleteAlarmInfo(@RequestParam(value = "id") String id){
        try{
            alarmInfoService.deleteAlarmInfo(id);
            return  HttpResult.ok();
        }catch (Exception e){
            return  HttpResult.error("删除出错!");
        }
    }

    @SysLogger(title="报警信息",optDesc = "查询报警信息")
    @PostMapping("/query")
    public HttpResult queryAlarmInfo(@RequestParam(value = "alarmTime",required = false) String alarmTime,
                                     @RequestParam(value = "alarmType",required = false) String alarmType,
                                     @RequestParam(value = "buildingId",required = false) String buildingId,
                                     @RequestParam(value = "dealState",required = false) String dealState,
                                     @RequestParam(value = "page",required = true) int page,
                                     @RequestParam(value = "limit",required = true) int limit){
        try{
            List<AlarmInfoEntity> list;
            list=alarmInfoService.queryAlarmInfo(alarmTime,alarmType,buildingId,dealState,page,limit);
            int count=alarmInfoService.getAlarmInfoCount(alarmTime,alarmType,buildingId,dealState,page,limit);
            return  HttpResult.ok(count,list);
        }catch (Exception e){
            e.printStackTrace();
            return  HttpResult.error("查询出错!");
        }
    }

    @SysLogger(title="报警设置",optDesc = "保存报警设置信息")
    @PostMapping("/set")
    public HttpResult saveAlarmValueSet(@RequestBody AlarmValueSetEntity alarmValueSetEntity){
        try{
            String type=alarmValueSetEntity.getType();
            String itemType=alarmValueSetEntity.getItemType();
            if (type==null || type.equals("") || type.equals("0")){
                String buildingId=alarmValueSetEntity.getBuildingId();
                alarmInfoService.delAlarmValueSet2(buildingId,itemType); //删除旧的记录
                alarmInfoService.saveAlarmValueSet(alarmValueSetEntity);
            }else{
              List<BuildingModel> list=buildingService.selectBuildingName();
              for ( BuildingModel build:list){
                  alarmInfoService.delAlarmValueSet2(String.valueOf(build.getBuildingId()),itemType);
                  alarmValueSetEntity.setBuildingId(String.valueOf(build.getBuildingId()));
                  alarmInfoService.saveAlarmValueSet(alarmValueSetEntity);
              }
            }
            return  HttpResult.ok();
        }catch (Exception e){
            return  HttpResult.error(500,"保存出错!");
        }
    }

    //修改处理状态
    @SysLogger(title="报警信息",optDesc = "修改处理状态")
    @PostMapping("/dealState")
    public HttpResult updateDealState(@RequestParam(value = "id",required = false) String id,
                                        @RequestParam(value = "dealState",required = false) String dealState){
        try{
            alarmInfoService.updateDealState(id,dealState);
            SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd");
            Date dt=new Date();
            String curDate=sdf1.format(dt);
            dealDataService.proAlarmInfoSum(curDate);  //汇总相关警报数
            return  HttpResult.ok();
        }catch (Exception e){
            return  HttpResult.error(500,"修改出错!");
        }
    }

    @SysLogger(title="报警设置",optDesc = "查询报警设置信息")
    @PostMapping("/querySet")
    public HttpResult queryAlarmValueSet(@RequestParam(value = "buildingId",required = false) String buildingId,
                                         @RequestParam(value = "itemType",required = false) String itemType,
                                         @RequestParam(value = "page",required = true) int page,
                                         @RequestParam(value = "limit",required = true) int limit){
        try{
            List<AlarmValueSetEntity> list;
            list=alarmInfoService.queryAlarmValueSet(buildingId,itemType,page,limit);
            int count=alarmInfoService.getAlarmValueSetCount(buildingId,itemType);
            return  HttpResult.ok(count,list);
        }catch (Exception e){
            e.printStackTrace();
            return  HttpResult.error("查询出错!");
        }
    }

    //删除设置记录
    @SysLogger(title="报警设置",optDesc = "删除报警设置信息")
    @PostMapping("/delSet")
    public HttpResult delAlarmValueSet(@RequestParam(value = "id",required = false) String id){
        try{
            alarmInfoService.delAlarmValueSet(id);
            return  HttpResult.ok();
        }catch (Exception e){
            return  HttpResult.error(500,"删除出错!");
        }

    }

    //按楼栋编号删除设置记录
    @SysLogger(title="报警设置",optDesc = "删除报警设置信息")
    @PostMapping("/del")
    public HttpResult delAlarmValueSet2(@RequestParam(value = "buildingId",required = false) String buildingId,
                                        @RequestParam(value = "itemType",required = false) String itemType){
        try{
            alarmInfoService.delAlarmValueSet2(buildingId,itemType);
            return  HttpResult.ok();
        }catch (Exception e){
            return  HttpResult.error(500,"删除出错!");
        }
    }
}
