package com.mh.user.controller;


import com.mh.common.http.HttpResult;
import com.mh.user.annotation.SysLogger;
import com.mh.user.entity.*;
import com.mh.user.model.DeviceModel;
import com.mh.user.service.BuildingService;
import com.mh.user.service.DeviceFloorService;
import com.mh.user.service.DeviceInstallService;
import org.apache.poi.hssf.usermodel.HSSFCell;
import org.apache.poi.hssf.usermodel.HSSFSheet;
import org.apache.poi.hssf.usermodel.HSSFWorkbook;
import org.apache.poi.ss.usermodel.CellType;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;

import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

@RestController
@RequestMapping("floor")
public class DeviceFloorController {

    @Autowired
    private DeviceFloorService deviceFloorService;

    @Autowired
    BuildingService buildingService;

    //保存
    @SysLogger(title="楼面设备",optDesc = "保存楼面设备信息")
    @PostMapping(value="/save")
    public HttpResult saveDevice(@RequestBody DeviceFloorEntity deviceFloorEntity) {
        return HttpResult.ok(deviceFloorService.saveDevice(deviceFloorEntity));
    }

    //修改
    @SysLogger(title="楼面设备",optDesc = "修改楼面设备信息")
    @PostMapping(value="/update")
    public HttpResult updateDevice(@RequestBody DeviceFloorEntity deviceFloorEntity) {
        return HttpResult.ok(deviceFloorService.updateDevice(deviceFloorEntity));
    }

    //查询所有
    @PostMapping(value = "/query")
    @SysLogger(title="楼面设备",optDesc = "查询楼面设备信息")
    public HttpResult queryDeviceFloor(@RequestParam(value = "buildingId", required = false)String buildingId,
                                       @RequestParam(value = "deviceType", required = false)String deviceType,
                                       @RequestParam(value= "page", required=true)Integer page,
                                       @RequestParam(value= "limit", required=true)Integer limit)  {
        try{
            int count=deviceFloorService.getCount(buildingId,deviceType, page,limit);
            List<DeviceFloorEntity> records=deviceFloorService.queryDeviceFloor(buildingId,deviceType, page,limit);
            return HttpResult.ok(count,records);
        }catch (Exception e){
            e.printStackTrace();
            return HttpResult.error();
        }
    }

    @PostMapping(value="/name")
    public HttpResult selectDeviceName(@RequestParam(value = "buildingId", required = false)String buildingId,
                                       @RequestParam(value = "deviceType", required = false)String deviceType) {
        try{
            List<DeviceModel> list=deviceFloorService.selectDeviceName(buildingId,deviceType);
            return HttpResult.ok(list);
        }catch (Exception e){
            e.printStackTrace();
            return HttpResult.error();
        }

    }

    // 删除多
    @PostMapping(value="/deletes")
    public HttpResult delete(@RequestBody List<DeviceFloorEntity> records) {
        return HttpResult.ok(deviceFloorService.deleteDevice(records));
    }

    // 删除单个
    @SysLogger(title="楼面设备",optDesc = "删除楼面设备信息")
    @PostMapping(value="/delete")
    public HttpResult delete(@RequestParam String id ) {
        return HttpResult.ok(deviceFloorService.deleteDevice(id));
    }

    // 资料批量上传
    @SysLogger(title="楼面设备",optDesc = "批量导入楼面设备信息")
    @PostMapping("/import_devices")
    public HttpResult importExcel(@RequestParam(value = "file") MultipartFile file, HttpServletRequest req) {
        SimpleDateFormat sdf = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");
        HttpResult httpResult=new HttpResult();
        try {
            int is=0;     //判断是否有重复表号
            String msg="";
            List<String> a = new ArrayList();
            InputStream inputStream = file.getInputStream();
            //创建工作簿
            HSSFWorkbook hssfWorkbook = new HSSFWorkbook(inputStream);
            System.out.println("xssfWorkbook对象：" + hssfWorkbook);
            //读取第一个工作表
            HSSFSheet sheet = hssfWorkbook.getSheetAt(0);
            System.out.println("sheet对象：" + sheet);
            //获取最后一行的num，即总行数。此处从0开始计数
            int maxRow = sheet.getLastRowNum();
            System.out.println("总行数为：" + maxRow);
            if (maxRow >= 500){
                msg = "总行数不能超出500行";
                httpResult.setMsg(msg);
                httpResult.setCode(500);
                return httpResult;
            }
            if (maxRow == 0){
                msg = "请先录入数据到excel文件";
                httpResult.setMsg(msg);
                httpResult.setCode(500);
                return httpResult;
            }

            // 创建数组集合
            List<DeviceFloorEntity> uploadEntityList = new ArrayList<>();
            List<String> deviceList = new ArrayList<>();
            for (int row = 1; row <= maxRow; row++) {
                //获取最后单元格num，即总单元格数 ***注意：此处从1开始计数***
                int maxRol = sheet.getRow(row).getLastCellNum();
                System.out.println("总列数为：" + maxRol);
                System.out.println("--------第" + row + "行的数据如下--------");
                for (int rol = 0; rol < maxRol; rol++){
                    String sCell;
                    if (sheet.getRow(row).getCell(rol) == null){
                        sCell="";
                    }else{
                        HSSFCell cell = sheet.getRow(row).getCell(rol);
                        cell.setCellType(CellType.STRING);
                        sCell = cell.getStringCellValue();
                    }
                    sCell = sCell.trim();                                     //去首尾空格
                    sCell = sCell.replaceAll(" ", "");      //去掉所有空格，包括首尾、中间
                    sCell = sCell.replaceAll("\\s*", "");   //可以替换大部分空白字符， 不限于空格,\s 可以匹配空格、制表符、换页符等空白字符的其中任意一个
                    System.out.print(sCell + "  ");
                    deviceList.add(sCell);
                    String rolName = "";
                    switch (rol){
                        case 1 :
                            rolName = "设备名称";
                            break;
                        case 2 :
                            rolName = "设备类型";
                            break;
                        case 3 :
                            rolName = "品牌";
                            break;
                        case 4 :
                            rolName = "规格";
                            break;
                        case 5 :
                            rolName = "型号";
                            break;
                        case 6 :
                            rolName = "安装人员";
                            break;
                        case 7 :
                            rolName = "所属楼栋";
                            break;
                    }
                    if ((rol >= 1)&&(rol <= 4)&&(sCell.equals(""))){
                        msg = rolName + "不能为空" ;
                        httpResult.setMsg(msg);
                        httpResult.setCode(500);
                        return httpResult;
                    }
                }

                // 创建实体类
                DeviceFloorEntity uploadEntity = new DeviceFloorEntity();
                uploadEntity.setDeviceName(deviceList.get(0));//设备名称
                uploadEntity.setDeviceType(deviceList.get(1));//设备类型
                uploadEntity.setBrand(deviceList.get(2));//品牌
                uploadEntity.setSpecs(deviceList.get(3));//规格
                uploadEntity.setModel(deviceList.get(4));//型号
                uploadEntity.setInstaller(deviceList.get(5));//安装人员
                uploadEntity.setBuildingId(deviceList.get(6));//所属楼栋
                deviceList.clear();
                uploadEntityList.add(uploadEntity);
                is=deviceFloorService.selectDeviceCount(uploadEntity.getBuildingId(),uploadEntity.getDeviceType(),uploadEntity.getDeviceName());
                if (is>0){
                    httpResult.setMsg("设备名称不能重复！");
                    httpResult.setCode(500);
//                    return httpResult;
                }
//                deviceFloorService.saveDevice(uploadEntity);
            }

            if (is==0){
                for (DeviceFloorEntity deviceFloorEntity:uploadEntityList){
                    deviceFloorService.saveDevice(deviceFloorEntity);
                }
                httpResult.setMsg("success");
                httpResult.setCode(200);
                return  httpResult;
            }
        } catch (IOException e) { // IOException | ParseException e
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return httpResult;
    }

}
