package com.mh.user.controller;

import com.alibaba.fastjson2.JSONObject;
import com.mh.common.http.HttpResult;
import com.mh.common.utils.StringUtils;
import com.mh.user.annotation.SysLogger;
import com.mh.user.constants.Constant;
import com.mh.user.entity.ControlSetEntity;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.entity.DeviceInstallEntity;
import com.mh.user.entity.PumpSetEntity;
import com.mh.user.model.DeviceModel;
import com.mh.user.model.SerialPortModel;
import com.mh.user.serialport.SerialPortSingle2;
import com.mh.user.service.ControlSetService;
import com.mh.user.service.DeviceInstallService;
import com.mh.user.service.NowDataService;
import com.mh.user.service.PumpSetService;
import com.mh.user.utils.ExchangeStringUtil;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.math.BigDecimal;
import java.util.List;
import java.util.Map;

@Slf4j
@RestController
@RequestMapping("serial")
public class SerialPortController {

    @Autowired
    DeviceInstallService deviceInstallService;

    @Autowired
    PumpSetService pumpSetService;

    @Autowired
    ControlSetService controlSetService;

    @Autowired
    NowDataService nowDataService;

    //操作设备
    @SysLogger(title = "控制管理", optDesc = "设置设备参数值")
    @PostMapping(value = "/operate/old")
    public HttpResult operateDevice(@RequestBody List<SerialPortModel> params) {
        try {
            SerialPortSingle2 serialPortSingle = new SerialPortSingle2(); //发送接收类
            DeviceCodeParamEntity deviceCodeParam = new DeviceCodeParamEntity();//参数实体类

            Constant.WEB_FLAG = true; //单抄,暂时停止采集
            for (SerialPortModel serialPortModel :
                    params) {
                String deviceAddr = serialPortModel.getDeviceAddr();//设备通讯地址
                String deviceType = serialPortModel.getDeviceType();//设备类型
                String buildingId = serialPortModel.getBuildingId();//楼栋
                String param = serialPortModel.getParam();//操作参数
                if (deviceAddr == null || deviceAddr.equals("")) {
                    List<DeviceModel> list = deviceInstallService.selectDevices(buildingId, deviceType);
                    deviceAddr = list.get(0).getDeviceAddr();
                }
                if (deviceAddr == null || deviceAddr.length() == 0) {
                    Constant.WEB_FLAG = false;
                    return HttpResult.error("通讯ID为空！");
                }
                DeviceInstallEntity deviceInstallEntity = deviceInstallService.selectDevice(deviceAddr, deviceType, buildingId);
                //发送指令实体类
                deviceCodeParam.setDeviceAddr(deviceAddr); //传入通讯编号
                deviceCodeParam.setDeviceType(deviceType);
                deviceCodeParam.setDataCom(deviceInstallEntity.getDataCom());
                deviceCodeParam.setBaudrate(deviceInstallEntity.getBaudRate());
                deviceCodeParam.setParity(deviceInstallEntity.getParity());
                deviceCodeParam.setDataValue(serialPortModel.getDataValue());//传入相关参数值
                deviceCodeParam.setBuildingId(buildingId);
                String brand = deviceInstallEntity.getBrand();//品牌
                deviceCodeParam.setBrand(brand);
                //设置设备实体对象
                ControlSetEntity controlData = new ControlSetEntity();
                controlData.setBuildingId(deviceInstallEntity.getBuildingId());
                if (deviceType == null || deviceType.equals("") || deviceType.equals("热泵")) {
                    //设置热泵实体对象
                    PumpSetEntity pumpData = new PumpSetEntity();
                    pumpData.setBuildingId(deviceInstallEntity.getBuildingId());
                    pumpData.setPumpId(deviceAddr);
                    if (param == null || param.equals("") || param.equals("温度设定")) {
                        //发送指令
                        if (brand.equals("美的")) {
                            deviceCodeParam.setRegisterAddr("0642"); //寄存器地址
                            deviceCodeParam.setFunCode("10");        //功能码写数据
                        } else {
                            deviceCodeParam.setRegisterAddr("0003"); //寄存器地址
                            deviceCodeParam.setFunCode("06");        //功能码写数据
                        }
                        //保存数据
                        pumpData.setTempSet(serialPortModel.getDataValue());
                        pumpSetService.savePumpSet(pumpData);//热泵信息保存数据库

                        nowDataService.upTempSet2(buildingId, serialPortModel.getDataValue(), deviceAddr);//更新实时状态表
                        log.info("楼栋编号：" + buildingId + ",设定温度：" + serialPortModel.getDataValue() + ",热泵编号：" + deviceAddr);
                    } else if (param.equals("时段1")) {
                        if (brand.equals("美的")) {
                            //发送指令
                            deviceCodeParam.setRegisterAddr("0656");    //寄存器地址
                            deviceCodeParam.setFunCode("10");           //功能码写数据
                            //保存数据
                            String time = serialPortModel.getDataValue();
                            if (time.length() == 8) {
                                String statTime = time.substring(0, 2) + ":" + time.substring(2, 4);
                                String closeTime = time.substring(4, 6) + ":" + time.substring(6, 8);
                                pumpData.setStartTime1(statTime);
                                pumpData.setCloseTime1(closeTime);
                            }
                            pumpSetService.savePumpSet(pumpData);//热泵信息保存数据库
                        }
                    } else if (param.equals("时段2")) {
                        if (brand.equals("美的")) {
                            //发送指令
                            deviceCodeParam.setRegisterAddr("065A");    //寄存器地址
                            deviceCodeParam.setFunCode("10");           //功能码写数据
                            //保存数据
                            String time = serialPortModel.getDataValue();
                            if (time.length() == 8) {
                                String statTime = time.substring(0, 2) + ":" + time.substring(2, 4);
                                String closeTime = time.substring(4, 6) + ":" + time.substring(6, 8);
                                pumpData.setStartTime2(statTime);
                                pumpData.setCloseTime2(closeTime);
                            }
                            pumpSetService.savePumpSet(pumpData);//热泵信息保存数据库
                        }
                    }
                } else if (deviceType.equals("时控")) {
                    deviceCodeParam.setFunCode("10");        //功能码写数据
                    String time = serialPortModel.getDataValue();
                    if (time.length() == 16) {
                        //时段1 HHmmHHmm
                        String statTime1 = time.substring(0, 2) + ":" + time.substring(2, 4);  //HH:mm
                        String closeTime1 = time.substring(4, 6) + ":" + time.substring(6, 8); //HH:mm
                        //时段2
                        String statTime2 = time.substring(8, 10) + ":" + time.substring(10, 12);
                        String closeTime2 = time.substring(12, 14) + ":" + time.substring(14, 16);
                        if (param.equals("L1")) {
                            deviceCodeParam.setRegisterAddr("0009"); //寄存器地址,L3路,L1(0009),L2(000D)
                            controlData.setUseStartTime1(statTime1);
                            controlData.setUseCloseTime1(closeTime1);
                            controlData.setUseStartTime2(statTime2);
                            controlData.setUseCloseTime2(closeTime2);
                        } else if (param.equals("L2")) {
                            deviceCodeParam.setRegisterAddr("000D");
                            controlData.setUseStartTime3(statTime1);
                            controlData.setUseCloseTime3(closeTime1);
                            controlData.setUseStartTime4(statTime2);
                            controlData.setUseCloseTime4(closeTime2);
                        } else {
                            deviceCodeParam.setRegisterAddr("0011");
                            controlData.setUseStartTime5(statTime1);
                            controlData.setUseCloseTime5(closeTime1);
                            controlData.setUseStartTime6(statTime2);
                            controlData.setUseCloseTime6(closeTime2);
                        }
                    }
                    controlSetService.saveControlSet(controlData); //保存设置内容
                } else if (deviceType.equals("水位开关")) {
                    if (brand == null || brand.equals("") || brand.equals("中凯")) {//品牌
                        deviceCodeParam.setFunCode("12");        //功能码写数据
                        if (!serialPortModel.getDataValue().equals("100%")) {
                            deviceCodeParam.setDataValue("100%");
                            serialPortSingle.serialPortSend(deviceCodeParam);//生成并发送指令
                            Thread.sleep(1000);
                        }
                    } else if (brand.equals("远向")) {
                        deviceCodeParam.setFunCode("06");        //功能码写数据
                        if (!serialPortModel.getDataValue().equals("100%")) {
                            deviceCodeParam.setDataValue("100%");
                            serialPortSingle.serialPortSend(deviceCodeParam);//生成并发送指令
                            Thread.sleep(1000);
                        }
                    } else if (brand.equals("顶威")) {
                        deviceCodeParam.setFunCode("0407");      //功能码写数据
                    }
                    deviceCodeParam.setDataValue(serialPortModel.getDataValue());
                    //controlData.setLevelSet(serialPortModel.getDataValue());
                    //controlSetService.saveControlSet(controlData);
                    nowDataService.upLevelSet(buildingId, serialPortModel.getDataValue());//更新实时状态表
                }
                String returnStr = serialPortSingle.serialPortSend(deviceCodeParam);//生成并发送指令
                if (!StringUtils.isBlank(returnStr) && "fail".equals(returnStr)) {
                    return HttpResult.error(500, "fail");
                }
            }
            Constant.WEB_FLAG = false; //单抄,恢复采集
            return HttpResult.ok();
        } catch (Exception e) {
            log.error("前端发送控制异常==>", e);
            Constant.WEB_FLAG = false; //单抄,恢复采集
            return HttpResult.error(500, "fail");
        } finally {
            Constant.WEB_FLAG = false;
        }
    }

    //读数据
    @SysLogger(title = "控制管理", optDesc = "读设备数据")
    @PostMapping(value = "/read/old")
    public HttpResult readData(@RequestBody List<SerialPortModel> params) {
        try {
            SerialPortSingle2 serialPortSingle = new SerialPortSingle2();
            DeviceCodeParamEntity deviceCodeParam = new DeviceCodeParamEntity();
            String rtData = ""; //返回值
            Constant.WEB_FLAG = true; //单抄,暂时停止采集
            for (SerialPortModel serialPortModel :
                    params) {
                String deviceAddr = serialPortModel.getDeviceAddr();//设备通讯地址
                String deviceType = serialPortModel.getDeviceType();//设备类型
                String buildingId = serialPortModel.getBuildingId();//楼栋
                String param = serialPortModel.getParam();//操作参数
                if (StringUtils.isBlank(deviceAddr)) {
                    List<DeviceModel> list = deviceInstallService.selectDevices(buildingId, deviceType);
                    deviceAddr = list.get(0).getDeviceAddr();
                }
                if (StringUtils.isBlank(deviceAddr)) {
                    Constant.WEB_FLAG = false;
                    return HttpResult.error("通讯ID为空！");
                }
                DeviceInstallEntity deviceInstallEntity = deviceInstallService.selectDevice(deviceAddr, deviceType, buildingId);
                //发送指令实体类
                deviceCodeParam.setDeviceAddr(deviceAddr);
                deviceCodeParam.setDeviceType(deviceType);
                deviceCodeParam.setDataCom(deviceInstallEntity.getDataCom());
                deviceCodeParam.setBaudrate(deviceInstallEntity.getBaudRate());
                deviceCodeParam.setParity(deviceInstallEntity.getParity());
                deviceCodeParam.setDataValue(serialPortModel.getDataValue());//传入相关参数值
                deviceCodeParam.setBuildingId(buildingId);
                String brand = deviceInstallEntity.getBrand();//品牌
                deviceCodeParam.setBrand(brand);
                //设置设备实体对象
                ControlSetEntity controlData = new ControlSetEntity();
                controlData.setBuildingId(deviceInstallEntity.getBuildingId());
                if (StringUtils.isBlank(deviceType) || deviceType.equals("热泵")) {
                    //设置热泵实体对象
                    PumpSetEntity pumpData = new PumpSetEntity();
                    pumpData.setBuildingId(deviceInstallEntity.getBuildingId());
                    pumpData.setPumpId(deviceAddr);
                    if (StringUtils.isBlank(param) || param.equals("读温度设定")) {
                        deviceCodeParam.setFunCode("03");            //功能码读数据
                        if (brand.equals("美的")) {
                            deviceCodeParam.setRegisterAddr("0642"); //寄存器地址
                        } else {
                            deviceCodeParam.setRegisterAddr("0003"); //寄存器地址
                        }
                        rtData = serialPortSingle.serialPortSend(deviceCodeParam);//生成并发送指令
                        if (!rtData.equals("fail")) {
                            pumpData.setTempSet(rtData);
                            pumpSetService.savePumpSet(pumpData);//热泵信息保存数据库
                        }
                    } else if (param.equals("实际温度")) {
                        deviceCodeParam.setFunCode("03");        //功能码读数据
                        if (brand.equals("美的")) {
                            deviceCodeParam.setRegisterAddr("0007"); //寄存器地址
                        } else {
                            deviceCodeParam.setRegisterAddr("0064"); //寄存器地址
                        }
                        rtData = serialPortSingle.serialPortSend(deviceCodeParam);//生成并发送指令
                        if (!rtData.equals("fail")) {
                            pumpData.setWaterTemp(rtData);
                            pumpSetService.savePumpSet(pumpData);//热泵信息保存数据库
                        }
                    } else if (param.equals("运行状态")) {
                        deviceCodeParam.setFunCode("03");        //功能码读数据
                        if (brand.equals("美的")) {
                            deviceCodeParam.setRegisterAddr("0641"); //寄存器地址
                        } else {
                            deviceCodeParam.setRegisterAddr("0BBD"); //寄存器地址
                        }
                        rtData = serialPortSingle.serialPortSend(deviceCodeParam);//生成并发送指令
                        //pumpData(rtData);
                        //pumpSetService.savePumpSet(pumpData);//热泵信息保存数据库
                    } else if (param.equals("时段1")) {
                        if (brand.equals("美的")) {
                            //发送指令
                            deviceCodeParam.setRegisterAddr("0656");    //寄存器地址
                            deviceCodeParam.setFunCode("03");           //功能码读数据
                            //保存数据
                            rtData = serialPortSingle.serialPortSend(deviceCodeParam);//生成并发送指令
                            if (!rtData.equals("fail")) {
                                if (rtData.length() == 8) {
                                    String statTime = rtData.substring(0, 2) + ":" + rtData.substring(2, 4);
                                    String closeTime = rtData.substring(4, 6) + ":" + rtData.substring(6, 8);
                                    pumpData.setStartTime1(statTime);
                                    pumpData.setCloseTime1(closeTime);
                                }
                                pumpSetService.savePumpSet(pumpData);//热泵信息保存数据库
                            }
                        }
                    } else if (param.equals("时段2")) {
                        if (brand.equals("美的")) {
                            //发送指令
                            deviceCodeParam.setRegisterAddr("065A");    //寄存器地址
                            deviceCodeParam.setFunCode("03");           //功能码读数据
                            //保存数据
                            rtData = serialPortSingle.serialPortSend(deviceCodeParam);//生成并发送指令
                            if (!rtData.equals("fail")) {
                                if (rtData.length() == 8) {
                                    String statTime = rtData.substring(0, 2) + ":" + rtData.substring(2, 4);
                                    String closeTime = rtData.substring(4, 6) + ":" + rtData.substring(6, 8);
                                    pumpData.setStartTime2(statTime);
                                    pumpData.setCloseTime2(closeTime);
                                }
                                pumpSetService.savePumpSet(pumpData);//热泵信息保存数据库
                            }
                        }
                    }
                } else if (deviceType.equals("时控")) {
                    if (param.equals("L1")) {
                        deviceCodeParam.setRegisterAddr("0009"); //寄存器地址,L3路,L1(0009),L2(000D)
                    } else if (param.equals("L2")) {
                        deviceCodeParam.setRegisterAddr("000D");
                    } else {
                        deviceCodeParam.setRegisterAddr("0011");
                    }
                    deviceCodeParam.setFunCode("03");        //功能码读数据
                    rtData = serialPortSingle.serialPortSend(deviceCodeParam);//生成并发送指令
                    if (!rtData.equals("fail")) {
                        if (rtData.length() == 16) {
                            //时段1
                            String statTime1 = rtData.substring(0, 2) + ":" + rtData.substring(2, 4);
                            String closeTime1 = rtData.substring(4, 6) + ":" + rtData.substring(6, 8);
                            //时段2
                            String statTime2 = rtData.substring(8, 10) + ":" + rtData.substring(10, 12);
                            String closeTime2 = rtData.substring(12, 14) + ":" + rtData.substring(14, 16);

                            if (param.equals("L1")) {
                                controlData.setUseStartTime1(statTime1);
                                controlData.setUseCloseTime1(closeTime1);
                                controlData.setUseStartTime2(statTime2);
                                controlData.setUseCloseTime2(closeTime2);
                            } else if (param.equals("L2")) {
                                controlData.setUseStartTime3(statTime1);
                                controlData.setUseCloseTime3(closeTime1);
                                controlData.setUseStartTime4(statTime2);
                                controlData.setUseCloseTime4(closeTime2);
                            } else {
                                controlData.setUseStartTime5(statTime1);
                                controlData.setUseCloseTime5(closeTime1);
                                controlData.setUseStartTime6(statTime2);
                                controlData.setUseCloseTime6(closeTime2);
                            }
                        }
                        controlSetService.saveControlSet(controlData); //保存设置内容
                    }
                } else if (deviceType.equals("水位开关") && !param.equals("状态")) {
                    if (brand == null || brand.equals("") || brand.equals("中凯")) {//品牌
                        deviceCodeParam.setFunCode("17");        //功能码读
                        deviceCodeParam.setRegisterAddr("0017");
                    } else if (brand.equals("远向")) {
                        deviceCodeParam.setFunCode("03");        //功能码读
                        deviceCodeParam.setRegisterAddr("0018");
                    } else if (brand.equals("顶威")) {
                        deviceCodeParam.setFunCode("0102");      //功能码读
                        deviceCodeParam.setRegisterAddr("0102");
                    }
                    rtData = serialPortSingle.serialPortSend(deviceCodeParam);//生成并发送指令
                    //deviceCodeParam.setDataValue(rtData);
                    //controlData.setLevelSet(serialPortModel.getDataValue());
                    //controlSetService.saveControlSet(controlData);
                    log.info("--------------读取返回数据：" + rtData + "------------------");
                } else if (deviceType.equals("水位开关") && param.equals("状态")) {
                    if (brand.equals("远向")) {
                        deviceCodeParam.setFunCode("03");        //功能码读
                        deviceCodeParam.setRegisterAddr("0010");
                    }
                    rtData = serialPortSingle.serialPortSend(deviceCodeParam);//生成并发送指令

                } else if (deviceType.equals("温度变送器")) {
                    deviceCodeParam.setFunCode("03");        //功能码读
                    deviceCodeParam.setRegisterAddr("0028");
                    rtData = serialPortSingle.serialPortSend(deviceCodeParam);//生成并发送指令
                } else if (deviceType.equals("压变")) {
                    deviceCodeParam.setFunCode("03");        //功能码读
                    deviceCodeParam.setRegisterAddr("0004");
                    rtData = serialPortSingle.serialPortSend(deviceCodeParam);//生成并发送指令
                } else if (deviceType.equals("水表")) {
                    rtData = serialPortSingle.serialPortSend(deviceCodeParam);
                    // 要添加校准值
                    Double deviationValue = deviceInstallService.selectDeviceDeviation(deviceCodeParam.getDeviceAddr(),deviceCodeParam.getDeviceType(), deviceCodeParam.getBuildingId());
                    if (null != deviationValue) {
                        rtData = new BigDecimal(rtData).add(BigDecimal.valueOf(deviationValue)).toString();
                    }
                } else if (deviceType.equals("电表")) {
                    rtData = serialPortSingle.serialPortSend(deviceCodeParam);
                }
            }
            Constant.WEB_FLAG = false; //单抄,恢复采集
            if (ExchangeStringUtil.getJSONType(rtData)) {
                Map map = JSONObject.parseObject(rtData);
                return HttpResult.ok("success", map);
            } else {
                if (rtData.equals("fail")) {
                    return HttpResult.error(500, "fail");
                } else {
                    return HttpResult.ok(rtData, rtData);
                }
            }
        } catch (Exception e) {
            Constant.WEB_FLAG = false; //单抄,恢复采集
            return HttpResult.error(500, "fail");
        } finally {
            Constant.WEB_FLAG = false; //单抄,恢复采集
        }
    }

    @PostMapping(value = "/pump")
    public HttpResult queryPumpSet(@RequestParam(value = "pumpId", defaultValue = "2") String pumpId,
                                   @RequestParam(value = "buildingId") String buildingId) {
        try {
            PumpSetEntity list = pumpSetService.queryPumpSet(pumpId, buildingId);
            return HttpResult.ok(list);
        } catch (Exception e) {
            //e.printStackTrace();
            return HttpResult.error();
        }

    }

    @PostMapping(value = "/control")
    public HttpResult queryControlSet(@RequestParam(value = "buildingId") String buildingId) {
        try {
            ControlSetEntity list = controlSetService.queryControlSet(buildingId);
            return HttpResult.ok(list);
        } catch (Exception e) {
            // e.printStackTrace();
            return HttpResult.error();
        }
    }
}
