package com.mh.user.controller;

import com.mh.common.http.HttpResult;
import com.mh.common.page.PageRequest;
import com.mh.user.annotation.SysLogger;
import com.mh.user.entity.SysParamEntity;
import com.mh.user.model.SysLog;
import com.mh.user.service.SysLogService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.web.bind.annotation.*;

import java.util.ArrayList;
import java.util.List;

/**
 * 日志控制器
 * @author ljf
 * @date 2020-04-25
 */
@RestController
@RequestMapping("log")
public class SysLogController {

	@Autowired
	private SysLogService sysLogService;

	@PostMapping(value="/findPage")
	public HttpResult findPage(@RequestBody PageRequest pageRequest) {
		return HttpResult.ok(sysLogService.findPage(pageRequest));
	}

	@SysLogger(title="日志",optDesc = "按操作员查询对应的操作日志")
    @PostMapping(value="/findLogs")
    public HttpResult findLogs(@RequestParam(value = "userName", required = false)String userName,
							   @RequestParam int page,
							   @RequestParam int limit) {
        try{
			List<SysLog> list=new ArrayList<SysLog>();
			list=sysLogService.findLogs(userName,page,limit);
			int count=sysLogService.findCount(userName,page,limit);

			return HttpResult.ok(count,list);
		}catch (Exception e){
        	e.printStackTrace();
            return HttpResult.error();
		}

    }

    // 查询日志详情
	@SysLogger(title="日志",optDesc = "查询日志详情")
	@PostMapping(value="/logInfo")
    public HttpResult logInfo(@RequestParam int id){
		try{
			SysLog sysLog=sysLogService.logInfo(id);
			return HttpResult.ok(sysLog);
		}catch (Exception e){
			e.printStackTrace();
			return HttpResult.error();
		}
	}

	// 查询系统参数
	@PostMapping(value="/sysParam")
	public HttpResult selectSysParam(){
		try{
			SysParamEntity sysParam=sysLogService.selectSysParam();
			return HttpResult.ok(sysParam);
		}catch (Exception e){
			e.printStackTrace();
			return HttpResult.error();
		}
	}
}
