package com.mh.user.mapper;

import com.mh.user.entity.DeviceManageEntity;
import com.mh.user.provide.DeviceManageProvider;
import org.apache.ibatis.annotations.*;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author ljf
 * @title ：
 * @description ：关于设备的持久化层，mapper层
 * @updateTime 2020-05-15
 * @throws ：
 */
@Component
@Mapper
public interface DeviceManageMapper {

    // 查询全部的设备信息
//    @Select("select id, device_code, device_num, collection_num, data_com, initial_value, price, param_id, param_name, last_value, last_date, communication_type, device_type, remark, grade from device_manage")
    @SelectProvider(type = DeviceManageProvider.class, method = "getDeviceByOthers")
    @Results({
            @Result(column = "id", property = "id"),
            @Result(column = "device_code", property = "deviceCode"),
            @Result(column = "device_num", property = "deviceNum"),
            @Result(column = "collection_num", property = "collectionNum"),
            @Result(column = "data_com", property = "dataCom"),
            @Result(column = "initial_value", property = "initialValue"),
            @Result(column = "device_rate", property = "deviceRate"),
            @Result(column = "device_caliber", property = "deviceCaliber"),
            @Result(column = "price", property = "price"),
            @Result(column = "param_id", property = "paramId"),
            @Result(column = "param_name", property = "paramName"),
            @Result(column = "last_value", property = "lastValue"),
            @Result(column = "last_date", property = "lastDates"),
            @Result(column = "communication_type", property = "communicationType"),
            @Result(column = "device_type", property = "deviceType"),
            @Result(column = "create_time", property = "createTimes"),
            @Result(column = "update_time", property = "updateTimes"),
            @Result(column = "remark", property = "remark"),
            @Result(column = "grade", property = "grade")
    })
    List<DeviceManageEntity> getDeviceByOther(@Param("deviceNum") String deviceNum);

    /**
     * 更新基表信息
     * @param deviceManageEntity
     */
    @Update("update device_manage set device_code=#{deviceCode},collection_num=#{collectionNum},data_com=#{dataCom},initial_value=#{initialValue}," +
            "last_value=#{lastValue},device_caliber=#{deviceCaliber},device_rate=#{deviceRate},param_name=#{paramName}," +
            "create_time=#{createTimes},update_time=#{updateTimes},device_type=#{deviceType} where device_num=#{deviceNum}")
    void updateBaseMeter(DeviceManageEntity deviceManageEntity);

    /**
     * 插入基表信息
     * @param deviceManageEntity
     */
    @Insert("insert into device_manage(device_code,device_num,collection_num,data_com,initial_value,last_value,device_caliber,device_rate," +
            "param_name,create_time,update_time,device_type) values(#{deviceCode},#{deviceNum},#{collectionNum},#{dataCom},#{initialValue}," +
            "#{lastValue},#{deviceCaliber},#{deviceRate},#{paramName},getDate(),#{updateTimes},#{deviceType})")
    void insertBaseMeter(DeviceManageEntity deviceManageEntity);







    @Select("select id, device_code, device_num, collection_num, data_com, initial_value, price, param_id, param_name, last_value, last_date, communication_type, device_type, remark, grade from device_manage")
    @Results({
            @Result(column = "id", property = "id"),
            @Result(column = "device_code", property = "deviceCode"),
            @Result(column = "device_num", property = "deviceNum"),
            @Result(column = "collection_num", property = "collectionNum"),
            @Result(column = "data_com", property = "dataCom"),
            @Result(column = "initial_value", property = "initialValue"),
            @Result(column = "price", property = "price"),
            @Result(column = "param_id", property = "paramId"),
            @Result(column = "param_name", property = "paramName"),
            @Result(column = "last_value", property = "lastValue"),
            @Result(column = "last_date", property = "lastDate"),
            @Result(column = "communication_type", property = "communicationType"),
            @Result(column = "device_type", property = "deviceType"),
            @Result(column = "remark", property = "remark"),
            @Result(column = "grade", property = "grade")
    })
    List<DeviceManageEntity> queryAllDevice();

    /**
     * 删除设备信息
     * @param deviceId
     * @param deviceType
     */
    @DeleteProvider(type = DeviceManageProvider.class, method = "deleteDeviceInfo")
    void deleteDeviceInfo(@Param("deviceId")Integer deviceId,@Param("deviceType")String deviceType);

    // 查询全部的设备信息
//    @Select("select id, device_code, device_num, collection_num, data_com, initial_value, price, param_id, param_name, last_value, last_date, communication_type, device_type, remark, grade from device_manage")
    @SelectProvider(type = DeviceManageProvider.class, method = "queryDeviceByOthers")
    @Results({
            @Result(column = "id", property = "id"),
            @Result(column = "device_code", property = "deviceCode"),
            @Result(column = "device_num", property = "deviceNum"),
            @Result(column = "collection_num", property = "collectionNum"),
            @Result(column = "data_com", property = "dataCom"),
            @Result(column = "initial_value", property = "initialValue"),
            @Result(column = "price", property = "price"),
            @Result(column = "param_id", property = "paramId"),
            @Result(column = "param_name", property = "paramName"),
            @Result(column = "last_value", property = "lastValue"),
            @Result(column = "last_date", property = "lastDate"),
            @Result(column = "communication_type", property = "communicationType"),
            @Result(column = "device_type", property = "deviceType"),
            @Result(column = "remark", property = "remark"),
            @Result(column = "grade", property = "grade")
    })
    List<DeviceManageEntity> queryDeviceByOther(@Param("page") int page, @Param("size") int size, @Param("deviceNum") String deviceNum);

    @SelectProvider(type = DeviceManageProvider.class, method = "queryDeviceByOthersCount")
    int queryDeviceByOtherCount(@Param("deviceNum") String deviceNum);

    @Select("<script>" +
      "select id, device_code, device_num, collection_num, data_com, initial_value, price, param_id, param_name, " +
            "last_value, last_date, communication_type, device_type, remark, grade from device_manage where 1 = 1 " +
      "<if test = 'deviceType != null'> " +
      " and param_id = #{deviceType}" +
      "</if>" +
      "</script>")
    @Results({
            @Result(column = "id", property = "id"),
            @Result(column = "device_code", property = "deviceCode"),
            @Result(column = "device_num", property = "deviceNum"),
            @Result(column = "collection_num", property = "collectionNum"),
            @Result(column = "data_com", property = "dataCom"),
            @Result(column = "initial_value", property = "initialValue"),
            @Result(column = "price", property = "price"),
            @Result(column = "param_id", property = "paramId"),
            @Result(column = "param_name", property = "paramName"),
            @Result(column = "last_value", property = "lastValue"),
            @Result(column = "last_date", property = "lastDate"),
            @Result(column = "communication_type", property = "communicationType"),
            @Result(column = "device_type", property = "deviceType"),
            @Result(column = "remark", property = "remark"),
            @Result(column = "grade", property = "grade")
    })
    List<DeviceManageEntity> queryDevicesByType(@Param("deviceType") String deviceType);

    @Select("select id, device_code, device_num, collection_num, data_com, initial_value, price, param_id, param_name, " +
            "last_value, last_date, communication_type, device_type, remark, grade from device_${tableName}")
    @Results({
            @Result(column = "id", property = "id"),
            @Result(column = "device_code", property = "deviceCode"),
            @Result(column = "device_num", property = "deviceNum"),
            @Result(column = "collection_num", property = "collectionNum"),
            @Result(column = "data_com", property = "dataCom"),
            @Result(column = "initial_value", property = "initialValue"),
            @Result(column = "price", property = "price"),
            @Result(column = "param_id", property = "paramId"),
            @Result(column = "param_name", property = "paramName"),
            @Result(column = "last_value", property = "lastValue"),
            @Result(column = "last_date", property = "lastDate"),
            @Result(column = "communication_type", property = "communicationType"),
            @Result(column = "device_type", property = "deviceType"),
            @Result(column = "remark", property = "remark"),
            @Result(column = "grade", property = "grade")
    })
    List<DeviceManageEntity> queryDevicesByType1(@Param("tableName") String tableName);

}
