package com.mh.user.mapper;

import com.mh.user.entity.DeviceManageEntity;
import com.mh.user.entity.DeviceParameterEntity;
import com.mh.user.provide.DeviceParamProvider;
import org.apache.ibatis.annotations.*;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * @author ljf
 * @title ：
 * @description ：设备参数持久化层
 * @updateTime 2020-05-21
 * @throws ：
 */
@Component
@Mapper
public interface DeviceParamMapper {

    @SelectProvider(type = DeviceParamProvider.class, method = "getBaseMeterParamList")
    @Results({
            @Result(column = "id", property = "id"),
            @Result(column = "param_id", property = "paramId"),
            @Result(column = "param_name", property = "paramName"),
            @Result(column = "param_baudrate", property = "paramBaudrate"),
            @Result(column = "checks", property = "checks"),
            @Result(column = "grade", property = "grade")
    })
    List<DeviceParameterEntity> getBaseMeterParamList(@Param("baseMeterType")String baseMeterType);

    @Update("update device_parameter set param_name=#{paramName},param_baudrate=#{paramBaudrate},grade=#{grade},checks=#{checks} " +
            "where param_id=#{paramId}")
    void updateBaseMeterParam(DeviceParameterEntity deviceManageEntity);

    @Insert("insert into device_parameter(param_id,param_name,param_baudrate,grade,checks) values(#{paramId},#{paramName}," +
            "#{paramBaudrate},#{grade},#{checks})")
    void insertBaseMeterParam(DeviceParameterEntity deviceManageEntity);




    // 添加网关设备
    @Insert("insert into device_parameter(param_id, param_name, param_baudrate, grade, checks)" +
            " values (#{paramId}, #{paramName}, #{paramBaudrate}, 1, #{checks})")
    void insertDeviceParam(DeviceParameterEntity deviceParameterEntity);

    // 通关网关ID删除设备
    @Delete("delete from device_parameter where id = #{id}")
    void deleteDeviceParamByID(@Param("id") Long id);

    // 根据网关设备ID查询网关对应信息
    @Select("select id, param_id, param_name, param_baudrate, grade, checks from device_parameter where param_id = #{paramId}")
    @Results({
            @Result(column = "id", property = "id"),
            @Result(column = "param_id", property = "paramId"),
            @Result(column = "param_name", property = "paramName"),
            @Result(column = "param_baudrate", property = "paramBaudrate"),
            @Result(column = "checks", property = "checks"),
            @Result(column = "grade", property = "grade")
    })
    DeviceParameterEntity queryParamById(int paramId);

    // 查询全部信息
    @Select("select id, param_id, param_name, param_baudrate, grade, checks from device_parameter")
    @Results({
            @Result(column = "id", property = "id"),
            @Result(column = "param_id", property = "paramId"),
            @Result(column = "param_name", property = "paramName"),
            @Result(column = "param_baudrate", property = "paramBaudrate"),
            @Result(column = "checks", property = "checks"),
            @Result(column = "grade", property = "grade")
    })
    List<DeviceParameterEntity> queryAll();

    // 根据页码和查询条件查询对应的设备信息
    @SelectProvider(type = DeviceParamProvider.class, method = "queryByOther")
    @Results({
            @Result(column = "id", property = "id"),
            @Result(column = "param_id", property = "paramId"),
            @Result(column = "param_name", property = "paramName"),
            @Result(column = "param_baudrate", property = "paramBaudrate"),
            @Result(column = "checks", property = "checks"),
            @Result(column = "grade", property = "grade")
    })
    List<DeviceParameterEntity> queryByOther(int page, int size, int paramId);

    // 查询对应的总数
    @SelectProvider(type = DeviceParamProvider.class, method = "queryByOtherCount")
    int queryByOtherCount(int page, int size, int paramId);

}
