package com.mh.user.mapper;

import com.mh.user.entity.DBEntity;
import com.mh.user.mapper.provider.AlarmInfoProvider;
import com.mh.user.mapper.provider.SysUserProvider;
import com.mh.user.model.SysRole;
import com.mh.user.model.SysUser;
import org.apache.ibatis.annotations.*;

import java.util.List;

/**
 * author: ljf
 * desc: 系统用户Mapper
 */
@Mapper
public interface SysUserMapper {

    /**
     * 根据用户名称查询数据库信息
     * @param userName
     * @return
     */
    @Select("select DB_Names,DB_UserName,DB_Pwd,DB_Port,DB_IP from sys_user S LEFT JOIN Tbl_Project P ON S.DBIds = P.ID where S.user_name=#{userName}")
    DBEntity queryDBInfo(@Param("userName") String userName);

    int deleteByPrimaryKey(Long id);

    int insert(SysUser record);

    int insertSelective(SysUser record);

    SysUser selectByPrimaryKey(Long id);

    int updateByPrimaryKeySelective(SysUser record);

    int updateByPrimaryKey(SysUser record);

    List<SysUser> findPage();

    SysUser findByName(@Param(value = "userName") String name);

	List<SysUser> findPageByName(@Param(value = "userName") String name);

	List<SysUser> findPageByNameAndEmail(@Param(value = "userName") String name, @Param(value = "email") String email);

    @Results(value = {
            @Result(property="id",column="id"),
            @Result(property="userName",column="user_name"),
            @Result(property="createBy",column="create_by"),
            @Result(property="createTime",column="create_time"),
            @Result(property ="loginTime",column ="login_time"),
            @Result(property="roleNames",column="role_name")
    })
    @SelectProvider(type = SysUserProvider.class,method = "queryUsers")
    List<SysUser> queryUsers(@Param(value = "userName") String userName,@Param(value = "page") int page, @Param(value = "limit") int limit);

    //查询所有记录数
    @SelectProvider(type = SysUserProvider.class,method = "getCount")
    int getCount(@Param(value = "userName") String userName,@Param(value = "page") int page,@Param(value = "limit") int limit);

    //根据id修改操作员密码
    @Update("update sys_user set password=#{password},salt=#{salt} where id=#{id}")
    void updatePassword(@Param(value = "password") String password,
                        @Param(value = "salt") String salt,
                        @Param(value = "id") String id);

    //根据操作员名称修改登录日期
    @Update("update sys_user set login_time=getDate() where user_name=#{userName}")
    void updateLoginTime(@Param(value = "userName") String userName);

}
