package com.mh.user.serialport;

import com.mh.common.utils.StringUtils;
import com.mh.user.constants.Constant;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.entity.SysParamEntity;
import com.mh.user.service.*;
import com.mh.user.utils.*;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationContext;
import purejavacomm.SerialPort;

import java.nio.ByteBuffer;
import java.nio.charset.StandardCharsets;
import java.util.Comparator;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author nxr
 * @title ：
 * @description ： 串口发送和接收处理,采集类
 * @updateTime 2022-08-10
 * @throws ：
 */
@Slf4j
public class SerialPortSendReceive2 {
    List<DeviceCodeParamEntity> deviceManageEntityList;
    // 调用service
    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    DeviceInstallService deviceInstallService = context.getBean(DeviceInstallService.class);
    NowDataService nowDataService = context.getBean(NowDataService.class);
    BuildingService buildingService = context.getBean(BuildingService.class);
    AnalysisReceiveOrder485 analysisReceiveOrder485 = new AnalysisReceiveOrder485();

    SysParamService sysParamService = context.getBean(SysParamService.class);

    public void serialPortSend(String sort, String thread) {
        SerialPort serialPort = null;
        CacheUtil cacheUtil = CacheUtil.getInstance();
        try {
            //生成对应的采集指令
            List<DeviceCodeParamEntity> deviceParamsByType = cacheUtil.getDeviceParamsByType(sort);
            deviceManageEntityList = deviceParamsByType
                    .parallelStream()
                    .filter(value -> value.getThread().equals(thread))
                    .sorted(Comparator.comparing(DeviceCodeParamEntity::getDataCom))
                    .collect(Collectors.toList());
            int size = deviceManageEntityList.size();
            for (int i = 0; i < size; i++) {
                //判断网页端是否有操作设备的
                if (Constant.WEB_FLAG) {
                    if (serialPort != null) {
                        SerialTool.closePort(serialPort);
                    }
                    log.info("有指令下发退出定时采集");
                    break;
                }
                //获取设备实际波特率
                int baudrate = deviceManageEntityList.get(i).getBaudrate();
                //获取设备实际校验位
                String parity = deviceManageEntityList.get(i).getParity();
                String comName = deviceManageEntityList.get(i).getDataCom();

                if (StringUtils.isBlank(comName)) {
                    log.info("-------------串口：" + comName + "不存在!-------------");
                    continue;
                }
                String deviceAddr = deviceManageEntityList.get(i).getDeviceAddr();
                String deviceType = deviceManageEntityList.get(i).getDeviceType();
                String registerAddr = deviceManageEntityList.get(i).getRegisterAddr();
                String brand = deviceManageEntityList.get(i).getBrand();//品牌
                String buildingId = deviceManageEntityList.get(i).getBuildingId();
                String buildingName = buildingService.queryBuildingName(buildingId); //查询楼栋名称
                try {
                    //传入对应的串口参数并打开串口
                    if (StringUtils.isBlank(parity) || parity.equalsIgnoreCase("none")) {
                        serialPort = SerialTool.openPort(comName, baudrate, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_NONE);
                    } else {
                        serialPort = SerialTool.openPort(comName, baudrate, SerialPort.DATABITS_8, SerialPort.STOPBITS_1, SerialPort.PARITY_EVEN);
                    }
                    if (null == serialPort) {
                        continue;
                    }
                    //向串口发送指令
                    String sendStr = SendOrderUtils.getSendStr(deviceManageEntityList.get(i));
                    SerialTool.sendToPort(SerialTool.HexString2Bytes(sendStr), serialPort, sendStr, deviceType);
                    if (deviceType.equals("热泵")) {
                        for (int j = 0; j < 4; j++) {
                            Thread.sleep(1000);
                            // 判断网页端是否有操作设备的
                            if (Constant.WEB_FLAG) {
                                SerialTool.closePort(serialPort);
                                log.info("有指令下发退出定时采集");
                                break;
                            }
                        }
                    } else {
                        Thread.sleep(2000);
                    }
                } catch (Exception e) {
                    if (null != serialPort) {
                        SerialTool.closePort(serialPort);
                        log.error("发送窗口数据异常==>", e);
                    }
                    continue;
                }
                
                //从串口读取数据
                byte[] bytes = SerialTool.readFromPort(serialPort);
                Date date1 = new Date();
                String dateStr = DateUtil.dateToString(date1, "yyyy-MM-dd HH:mm:ss");
                if (bytes == null) {
                    SerialTool.closePort(serialPort);
                    log.info("串口" + serialPort.getName() + "没有数据返回！" + i);
                    log.info("----------------" + deviceType + "离线,设备号：" + deviceAddr + ",所属楼栋：" + buildingName + "----------------");
                    String time1 = deviceInstallService.selectLastDate(deviceType, deviceAddr, buildingId);
                    if (time1 == null) {
                        time1 = dateStr;
                    }
                    int d = ExchangeStringUtil.compareCopyTime(time1, dateStr);
                    if (d == 1) {
                        deviceInstallService.updateNotOnline(deviceAddr, deviceType, buildingId, "离线"); //所有设备离线
                        if (deviceType.equals("热泵")) {
                            nowDataService.updateRunState(buildingId, deviceAddr, "离线", buildingName); //监控界面状态表热泵在线状态
                        }
                    }
                    continue;
                }
                // 处理返回来的数据报文
                dealReceiveData(dateStr, serialPort, i, deviceAddr, deviceType, registerAddr, brand, buildingId, buildingName, bytes);
                try {
                    SerialTool.closePort(serialPort);
                    log.info("关闭串口==" + serialPort.getName());
                } catch (Exception e) {
                    log.error("关闭串口失败！", e);
                }
            }
        } catch (Exception e) {
            log.error("-------------串口采集异常！----------->>", e);
        }
    }

    /**
     * 处理返回来的数据
     * @param dateStr
     * @param serialPort
     * @param i
     * @param deviceAddr
     * @param deviceType
     * @param registerAddr
     * @param brand
     * @param buildingId
     * @param buildingName
     * @param bytes
     * @throws InterruptedException
     */
    private void dealReceiveData(String dateStr,
                                 SerialPort serialPort,
                                 int i,
                                 String deviceAddr,
                                 String deviceType,
                                 String registerAddr,
                                 String brand,
                                 String buildingId,
                                 String buildingName, byte[] bytes) {
        try {
            String receiveStr = "";
            receiveStr = ExchangeStringUtil.parseByte2HexStr(bytes);
            //去掉空格和null
            receiveStr = receiveStr.replace("null", "");
            receiveStr = receiveStr.replace(" ", "");
            log.info("串口" + serialPort + "接受第" + i + "数据：" + receiveStr + "，大小： " + receiveStr.length());
            //返回值全部变成大写
            String receiveData = receiveStr.toUpperCase();
            //截取去掉FE
            String dataStr;
            if (receiveData.length() > 8) {
                String str1 = receiveData.substring(0, 8);
                String str2 = receiveData.substring(8);
                dataStr = str1.replace("FE", "") + str2;
            } else {
                dataStr = receiveData.replace("FE", "");
            }
            deviceInstallService.updateOnline(deviceAddr, deviceType, buildingId, "在线"); //设备在线
            log.info("----------------" + deviceType + "在线,设备号：" + deviceAddr + ",所属楼栋：" + buildingName + "----------------");
            if (deviceType.equals("热泵")) {
                String strState = nowDataService.selectState(buildingId, deviceAddr);
                if (strState != null && strState.equals("离线")) { //采集到数据
                    nowDataService.updateRunState(buildingId, deviceAddr, "不运行", buildingName); //监控界面状态表热泵在线状态
                }
            }
            // 解析返回来的数据
            analysisReceiveData(dateStr, deviceType, registerAddr, brand, buildingId, dataStr, analysisReceiveOrder485, nowDataService, buildingName);
        } catch (Exception e) {
            log.error("楼栋：" + buildingName + "设备类型：" + deviceType + "保存数据库失败！" + i,e);
        }
    }

    public void analysisReceiveData(String dateStr,
                                    String deviceType,
                                    String registerAddr,
                                    String brand,
                                    String buildingId,
                                    String dataStr,
                                    AnalysisReceiveOrder485 analysisReceiveOrder485,
                                    NowDataService nowDataService,
                                    String buildingName) {
        if ((dataStr.length() == 36 || dataStr.length() == 44 || dataStr.length() == 40 || dataStr.length() == 50) && deviceType.equals("电表")) {
            analysisReceiveOrder485.analysisMeterOrder485(dataStr, registerAddr, brand, buildingId, buildingName);
            nowDataService.proWaterLevel(dateStr, buildingId, ""); //保存时间点楼栋水位
        } else if ((dataStr.length() == 18 || dataStr.length() == 70 || dataStr.length() == 44) && deviceType.equals("水表")) {
            analysisReceiveOrder485.analysisWtMeterOrder485(dataStr, registerAddr, brand, buildingId, buildingName);
        } else if (deviceType.equals("压变")) {
            analysisReceiveOrder485.analysisPressureOrder485(dateStr ,dataStr, registerAddr, brand, buildingId, buildingName);
        } else if (deviceType.equals("热泵")) {
            analysisReceiveOrder485.analysisPumpOrder485(dateStr ,dataStr, registerAddr, brand, buildingId, buildingName);
        } else if (deviceType.equals("温控")) {
            analysisReceiveOrder485.analysisTempOrder485(dataStr, registerAddr, brand, buildingId, buildingName);
        } else if (deviceType.equals("时控")) {
            analysisReceiveOrder485.analysisTimeSetOrder485(dateStr,dataStr, registerAddr, brand, buildingId, buildingName);
        } else if (deviceType.equals("水位开关") && (Constant.REG_ADDR_0018.equals(registerAddr) || Constant.REG_ADDR_0017.equals(registerAddr))) {
            SysParamEntity sysParamEntity = sysParamService.selectSysParam();
            String customName = sysParamEntity.getCustomName();
            analysisReceiveOrder485.analysisRelayOrder485(dateStr ,dataStr, registerAddr, brand, buildingId, buildingName, customName);
        } else if (dataStr.length() == 30 && deviceType.equals("状态检测")) {//五路状态读取,兼容旧版系统
            analysisReceiveOrder485.analysisStateOrder485(dataStr, registerAddr, brand, buildingId, buildingName);
        } else if (deviceType.equals("水位开关") && Constant.REG_ADDR_0010.equals(registerAddr)) {
            handleWaterSwitch(dateStr, dataStr, registerAddr, brand, buildingId, buildingName, analysisReceiveOrder485);
        } else if (deviceType.equals("温度变送器")) {
            analysisReceiveOrder485.analysisMulTempOrder485(dateStr, dataStr, registerAddr, brand, buildingId, buildingName);
        } else if (deviceType.equals("热泵状态")) {
            analysisReceiveOrder485.analysisPumpStateOrder(dateStr, dataStr, registerAddr, brand, buildingId, buildingName);
        }
    }

    private void handleWaterSwitch(String dateStr, String dataStr, String registerAddr, String brand,
                                   String buildingId, String buildingName, AnalysisReceiveOrder485 analysisReceiveOrder485) {
        log.info("进入水位开关，地址==>{}", registerAddr);
        SysParamEntity sysParamEntity = sysParamService.selectSysParam();
        String customName = sysParamEntity.getCustomName();
        if (!StringUtils.isBlank(customName)) {
            if (customName.contains(Constant.CUSTOM_NAME_HUAXIA)
                    || customName.contains(Constant.CUSTOM_NAME_GUANGSHANG)) {
                analysisReceiveOrder485.analysisRelayOrder485(dateStr, dataStr,registerAddr,brand,buildingId, buildingName, customName);
            } else {
                analysisReceiveOrder485.analysisPumpStateOrder(dateStr, dataStr, registerAddr, brand, buildingId, buildingName);
            }
        } else {
            analysisReceiveOrder485.analysisPumpStateOrder(dateStr, dataStr, registerAddr, brand, buildingId, buildingName);
        }
    }

    /**
     * 十六进制字符串转byte[]
     *
     * @param hex 十六进制字符串
     * @return byte[]
     */
    public static byte[] hexStr2Byte(String hex) {
        if (hex == null) {
            return new byte[]{};
        }

        // 奇数位补0
        if (hex.length() % 2 != 0) {
            hex = "0" + hex;
        }

        int length = hex.length();
        ByteBuffer buffer = ByteBuffer.allocate(length / 2);
        for (int i = 0; i < length; i++) {
            String hexStr = hex.charAt(i) + "";
            i++;
            hexStr += hex.charAt(i);
            byte b = (byte) Integer.parseInt(hexStr, 16);
            buffer.put(b);
        }
        return buffer.array();
    }

    /**
     * 16进制转换成为string类型字符串
     *
     * @param s 待转换字符串
     */
    public static String hexStringToString(String s) {
        if (s == null || "".equals(s)) {
            return null;
        }
        s = s.replace(" ", "");
        byte[] baKeyword = new byte[s.length() / 2];
        for (int i = 0; i < baKeyword.length; i++) {
            try {
                baKeyword[i] = (byte) (0xff & Integer.parseInt(s.substring(i * 2, i * 2 + 2), 16));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        try {
            s = new String(baKeyword, StandardCharsets.UTF_8);
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        return s;
    }
}
