package com.mh.user.service.impl;

import com.mh.user.dto.DeviceMessageDTO;
import com.mh.user.dto.DisplayBoxDataDTO;
import com.mh.user.entity.GatewayManageEntity;
import com.mh.user.mapper.DeviceDisplayMapper;
import com.mh.user.service.DeviceDisplayService;
import org.springframework.stereotype.Service;

import java.util.List;

/**
 * @author ljf
 * @title ：
 * @description ：  设备数据显示实现类
 * @updateTime 2020-05-29
 * @throws ：
 */
@Service
public class DeviceDisplayServiceImpl implements DeviceDisplayService {

    // 通过构造函数调用Mapper层接口
    private final DeviceDisplayMapper deviceDisplayMapper;

    public DeviceDisplayServiceImpl(DeviceDisplayMapper deviceDisplayMapper) {
        this.deviceDisplayMapper = deviceDisplayMapper;
    }

    @Override
    public List<DisplayBoxDataDTO> deviceDisplayBoxData(String deviceType, String type) {
        return deviceDisplayMapper.deviceDisplayBoxData(deviceType, type);
    }

    @Override
    public List<DeviceMessageDTO> queryDeviceStatus(String deviceType) {
        return deviceDisplayMapper.queryDeviceStatus(deviceType);
    }

    /**
     * 网关管理服务类
     * author：ljf
     * create—date：2020-05-21
     */
    public static interface GatewayManageService {

        /**
         * 根据条件查询网关信息
         * @param grade
         * @param operator
         * @return
         */
        List<GatewayManageEntity> queryByOther(Integer grade, Integer operator);

        /**
         * 新增或更新网关信息
         * @param gatewayManageEntity
         */
        void addOrUpdateGateWayInfo(GatewayManageEntity gatewayManageEntity);

        // 查询全部
        List<GatewayManageEntity> queryAll(String gatewayID, String operator, String grade, int page, int limit);

        // 查询记录数
        int queryCount(String gatewayID, String operator, String grade);

        // 添加网关设备
        String insertGatewayManage(GatewayManageEntity gatewayManageEntity);

        // 根据网关ID删除网关设备
        void deleteGatewayManageByID(int gatewayID);

        // 根据网关ID查询设备信息
        GatewayManageEntity queryGatewayByID(Long gatewayID);

        // 设备总数
        int queryByOtherCount(int page, int size, int gatewayID);

        // 更新网关管理器的最新连接数据
        void updateGatewayManage(String IP, String port);

        // 根据grade查询对应的网关路由信息
        GatewayManageEntity queryGatewayByGrade(Long grade);

    }
}
