package com.mh.user.service.impl;

import com.mh.user.entity.NowPublicDataEntity;
import com.mh.user.mapper.BuildingMapper;
import com.mh.user.mapper.DeviceInstallMapper;
import com.mh.user.mapper.NowPublicDataMapper;
import com.mh.user.model.TempModel;
import com.mh.user.service.NowPublicDataService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.List;

@Slf4j
@Service
public class NowPublicDataServiceImpl implements NowPublicDataService {

    @Autowired
    NowPublicDataMapper nowPublicDataMapper;

    @Autowired
    DeviceInstallMapper deviceInstallMapper;

    @Autowired
    BuildingMapper buildingMapper;

    private final SimpleDateFormat sdf1 = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    @Override
    public void saveNowPublicData(NowPublicDataEntity nowPublicDataEntity) {
        nowPublicDataMapper.saveNowPublicData(nowPublicDataEntity);
    }

    @Override
    public void updateNowPublicData(NowPublicDataEntity nowPublicDataEntity) {
        nowPublicDataMapper.updateNowPublicData(nowPublicDataEntity);
    }

    @Override
    public NowPublicDataEntity queryNowPublicData(String buildingId) {
        return nowPublicDataMapper.queryNowPublicData(buildingId);
    }

    @Override
    public void saveHistoryPublicData(NowPublicDataEntity nowPublicDataEntity) {
       nowPublicDataMapper.saveHistoryPublicData(nowPublicDataEntity);
    }

    @Override
    public void updateHistoryPublicData(NowPublicDataEntity nowPublicDataEntity) {
       nowPublicDataMapper.updateHistoryPublicData(nowPublicDataEntity);
    }

    @Override
    public List<NowPublicDataEntity> queryHistoryPublicData(String curDate, String buildingId, int page, int limit) {
        return nowPublicDataMapper.queryHistoryPublicData(curDate,buildingId,page,limit);
    }

    @Override
    public int getHistoryPublicDataCount(String curDate, String buildingId, int page, int limit) {
        return nowPublicDataMapper.getHistoryPublicDataCount(curDate,buildingId,page,limit);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public void saveNowHistoryPublicData(NowPublicDataEntity nowPublicDataEntity ) {
        Date date=new Date();
        String strDate=sdf1.format(date);
        strDate=strDate.substring(0,13)+":00:00";

        try{
            //从安装表获取设备信息
            String buildingId=nowPublicDataEntity.getBuildingId();
            String buildingName=buildingMapper.queryBuildingName(buildingId);
            nowPublicDataEntity.setBuildingName(buildingName);
//            DeviceInstallEntity deviceInstallEntity=deviceInstallMapper.selectDevice(deviceAddr,deviceType,buildingId);
            int r1 = nowPublicDataMapper.selectNowPublicDataCount(buildingId);//判断now_public_data表是否存在记录
            nowPublicDataEntity.setCurDate(sdf1.parse(strDate));

            if (r1==0){
                nowPublicDataMapper.saveNowPublicData(nowPublicDataEntity);
            } else {
                nowPublicDataMapper.updateNowPublicData(nowPublicDataEntity);
            }
            //判断历史表是否有记录
            int r2 = nowPublicDataMapper.selectHistoryPublicDataCount(strDate,buildingId);//判断now_data表是否存在记录
            if (r2==0){
                nowPublicDataMapper.saveHistoryPublicData(nowPublicDataEntity);
            }else {
                nowPublicDataMapper.updateHistoryPublicData(nowPublicDataEntity);
            }
        }catch (Exception e){
            log.error("保存历史公共数据失败",e);
        }
    }

    //查询楼栋水箱平均温度
    @Override
    public List<TempModel> queryWtTemp() {
        return nowPublicDataMapper.queryWtTemp();
    }

    //查询单楼栋水箱平均温度
    @Override
    public TempModel queryWtTemp2(String buildingId) {
        return nowPublicDataMapper.queryWtTemp2(buildingId);
    }

    //更新单箱温度
    @Override
    public void updateSingleTemp(String singleTemp, String buildingId) {
        nowPublicDataMapper.updateSingleTemp(singleTemp, buildingId);
    }
}
