package com.mh.user.utils;

import com.alibaba.fastjson2.JSONArray;
import com.alibaba.fastjson2.JSONObject;
import com.github.benmanes.caffeine.cache.Cache;
import com.mh.user.constants.Constant;
import com.mh.user.entity.ChillersEntity;
import com.mh.user.entity.DeviceCodeParamEntity;
import com.mh.user.service.DeviceCodeParamService;
import org.springframework.context.ApplicationContext;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * @Classname CacheUtil
 * Todo: 缓存工具类
 * @Date 2021-11-21 10:03
 * @Created by LJF
 */
public class CacheUtil {

    // 调用service
    ApplicationContext context = SpringBeanUtil.getApplicationContext();
    Cache caffeineCache = (Cache) context.getBean("caffeineCache");

    DeviceCodeParamService deviceCodeParamService = context.getBean(DeviceCodeParamService.class);

    private static class SingletonHolder{
        private static final CacheUtil instance=new CacheUtil();
    }
    private CacheUtil(){
        createDeviceParams();
    }
    public static CacheUtil getInstance(){
        return SingletonHolder.instance;
    }

    /**
     * 把采集ddc设置的报文存入到缓存中
     */
    private void createDeviceParams() {
        // 缓存压变、温控
        if (caffeineCache.getIfPresent("1") == null) {
            List<DeviceCodeParamEntity> deviceCodeParamEntities = deviceCodeParamService.queryCodeParam3(null);
            caffeineCache.put("1", deviceCodeParamEntities);
        }
        // 缓存水、电表、状态检测
        if (caffeineCache.getIfPresent("2") == null) {
            List<DeviceCodeParamEntity> deviceCodeParam2Entities = deviceCodeParamService.queryCodeParam4(null);
            caffeineCache.put("2", deviceCodeParam2Entities);
        }
        // 水位开关
        if (caffeineCache.getIfPresent("3") == null) {
            List<DeviceCodeParamEntity> deviceCodeParam3Entities = deviceCodeParamService.queryCodeParam5(null);
            caffeineCache.put("3", deviceCodeParam3Entities);
        }
    }

    /**
     * 通过发生类型获取发生的报文集合(返回List类型)
     * @param type
     * @return
     */
    public List<DeviceCodeParamEntity> getDeviceParamsByType(String type) {
        Object cacheObject = caffeineCache.getIfPresent(type);
        // 如果为空，重新添加
        if (cacheObject == null) {
            createDeviceParams();
            // 在重新获取数据
            cacheObject = caffeineCache.getIfPresent(type);
        }
        return JSONArray.parseArray(JSONObject.toJSONString(cacheObject), DeviceCodeParamEntity.class);
    }

    /**
     * 删除缓存
     */
    public void deleteCache() {
        caffeineCache.invalidateAll();
    }


}
